@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2012 Alexandru Munteanu - <io_fx@yahoo.fr>
+ * Copyright (c) 2005-2013 Alexandru Munteanu - <m@ioalex.net>
  *
  * Parts of this file have been copied from the 'vcut' 1.6
  * program provided with 'vorbis-tools' :
@@ -31,8 +31,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307,
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
  * USA.
  *
  *********************************************************/
@@ -58,8 +57,10 @@ splt_ogg_new_stream_handler *splt_ogg_nsh_new(splt_state *state,
     splt_ogg_state *oggstate, ogg_stream_state *stream_out, const char *output_fname,
     int write_header_packets, ogg_stream_state *optional_stream_in);
 
+void splt_ogg_nsh_free(splt_ogg_new_stream_handler **nsh);
+
 void splt_ogg_initialise_for_new_stream(splt_ogg_new_stream_handler *nsh, 
-    ogg_page *page, long *cutpoint, long previous_granulepos);
+    ogg_page *page, ogg_int64_t *cutpoint, ogg_int64_t previous_granulepos);
 int splt_ogg_new_stream_needs_header_packet();
 void splt_ogg_new_stream_handle_header_packet(splt_ogg_new_stream_handler *nsh, ogg_packet *packet, int *error);
 
