@@ -3,7 +3,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2012 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2013 Alexandru Munteanu - m@ioalex.net
  *
  *********************************************************/
 
@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
  * 02111-1307, USA.
  *********************************************************/
 
@@ -36,10 +36,12 @@ All that is needed in order to be able to read and write cue files.
 
 #include "cue.h"
 
+static void splt_cue_cu_free(cue_utils **cu);
+
 //! Process the rest of a cue line that begins with the word TRACK
 static void splt_cue_process_track_line(char *line_content, cue_utils *cu, splt_state *state)
 {
-  // Skip the word TRACK
+  //skip the word TRACK
   line_content += 5;
 
   if (cu->tracks == -1) 
@@ -53,13 +55,17 @@ static void splt_cue_process_track_line(char *line_content, cue_utils *cu, splt_
     cu->error = SPLT_INVALID_CUE_FILE;
   }
 
-  cu->performer = SPLT_FALSE;
-  cu->title = SPLT_FALSE;
   cu->time_for_track = SPLT_FALSE;
   cu->tracks++;
   cu->current_track_type = SPLT_SPLITPOINT;
 
   splt_tu_new_tags_if_necessary(state, cu->tracks - 1);
+
+  if (splt_o_get_int_option(state, SPLT_OPT_CUE_CDDB_ADD_TAGS_WITH_KEEP_ORIGINAL_TAGS))
+  {
+    int true_value = SPLT_TRUE;
+    splt_tu_set_tags_field(state, cu->tracks - 1, SPLT_TAGS_ORIGINAL, &true_value);
+  }
 }
 
 static void remove_trailing_spaces_and_quote(char *ptr_e, char *in)
@@ -115,30 +121,32 @@ static const char *splt_cue_parse_value(char *in, int skip_last_word)
   return ptr_b;
 }
 
+static char *splt_cue_parse_and_duplicate_value(char *in, int *error)
+{
+  const char *ptr_b = splt_cue_parse_value(in, SPLT_FALSE);
+
+  char *out = NULL;
+  int err = splt_su_append(&out, ptr_b, strlen(ptr_b) + 1, NULL);
+  if (err < 0)
+  {
+    *error = err;
+  }
+
+  return out;
+}
+
 //! Update data for a track in the central splt_state structure.
-static int splt_cue_store_value(splt_state *state, char *in,
-    int index, int tag_field)
+static int splt_cue_store_value(splt_state *state, char *in, int index, int tag_field, cue_utils *cu)
 {
   if (!in)
   {
     return SPLT_OK;
   }
 
-  const char *ptr_b = splt_cue_parse_value(in, SPLT_FALSE);
-
-  char *out = NULL;
-  int error = splt_su_append(&out, ptr_b, strlen(ptr_b) + 1, NULL);
+  splt_code error = SPLT_OK;
+  char *out = splt_cue_parse_and_duplicate_value(in, &error);
   if (error < 0) { return error; }
 
-  if (tag_field == SPLT_TAGS_ARTIST)
-  {
-    splt_c_put_info_message_to_client(state, _("\n  Artist: %s\n"), out);
-  }
-  else if (tag_field == SPLT_TAGS_ALBUM)
-  {
-    splt_c_put_info_message_to_client(state, _("  Album: %s\n"), out);
-  }
-
   error = splt_tu_set_tags_field(state, index, tag_field, out);
   if (out)
   {
@@ -154,31 +162,32 @@ static void splt_cue_process_title_line(char *line_content, cue_utils *cu, splt_
 {
   int err = SPLT_OK;
 
-  // Skip the word TITLE
+  //skip the word TITLE
   line_content += 5;
 
   if (cu->tracks == -1)
   {
-    if ((err = splt_cue_store_value(state, line_content, 
-            0, SPLT_TAGS_ALBUM)) != SPLT_OK)
+    char *album = splt_cue_parse_and_duplicate_value(line_content, &err);
+    if (err < 0) { cu->error = err; return; }
+
+    if (album)
     {
-      cu->error = err;
-      return;
+      splt_c_put_info_message_to_client(state, _("  Album: %s\n"), album);
+      err = splt_tu_set_field_on_tags(cu->all_tags, SPLT_TAGS_ALBUM, album);
+      free(album);
+      if (err < 0) { cu->error = err; return; }
     }
   }
   else
   {
     if (cu->tracks > 0)
     {
-      if ((err = splt_cue_store_value(state, line_content,
-              cu->tracks-1, SPLT_TAGS_TITLE)) != SPLT_OK)
+      if ((err = splt_cue_store_value(state, line_content, cu->tracks-1, SPLT_TAGS_TITLE, cu)) != SPLT_OK)
       {
         cu->error = err;
         return;
       }
     }
-
-    cu->title = SPLT_TRUE;
   }
 }
 
@@ -187,38 +196,41 @@ static void splt_cue_process_performer_line(char *line_content, cue_utils *cu, s
 {
   int err = SPLT_OK;
 
-  // Skip the word PERFORMER
+  //skip the word PERFORMER
   line_content += 9;
 
   if (cu->tracks == -1)
   {
-    //we always have one artist in a cue file, we
-    //put the performers if more than one artist
-    if ((err = splt_cue_store_value(state, line_content,
-            0, SPLT_TAGS_ARTIST)) != SPLT_OK)
+    char *artist = splt_cue_parse_and_duplicate_value(line_content, &err);
+    if (err < 0) { cu->error = err; return; }
+
+    if (artist)
     {
-      cu->error = err;
-      return;
+      splt_c_put_info_message_to_client(state, _("\n  Artist: %s\n"), artist);
+      err = splt_tu_set_field_on_tags(cu->all_tags, SPLT_TAGS_ARTIST, artist);
+      free(artist);
+      if (err < 0) { cu->error = err; return; }
     }
   }
   else
   {
     if (cu->tracks > 0)
     {
-      if ((err = splt_cue_store_value(state, line_content,
-              cu->tracks - 1, SPLT_TAGS_PERFORMER)) != SPLT_OK)
+      if ((err = splt_cue_store_value(state, line_content, cu->tracks - 1, SPLT_TAGS_ARTIST, cu)) != SPLT_OK)
+      {
+        cu->error = err;
+        return;
+      }
+
+      if ((err = splt_cue_store_value(state, line_content, cu->tracks - 1, SPLT_TAGS_PERFORMER, cu)) != SPLT_OK)
       {
         cu->error = err;
         return;
       }
     }
- 
-    cu->performer = SPLT_TRUE;
   }
-
 }
 
-
 //! Process the rest of a cue line that begins with the word INDEX
 static void splt_cue_process_index_line(char *line_content, cue_utils *cu, splt_state *state)
 {
@@ -241,10 +253,15 @@ static void splt_cue_process_index_line(char *line_content, cue_utils *cu, splt_
     return;
   }
 
-  err = splt_sp_append_splitpoint(state, hundr_seconds, NULL, 
-				  cu->current_track_type);
+  err = splt_sp_append_splitpoint(state, hundr_seconds, cu->current_name, cu->current_track_type);
   if (err < 0) { cu->error = err; return; }
 
+  if (cu->current_name)
+  {
+    free(cu->current_name);
+    cu->current_name = NULL;
+  }
+
   cu->time_for_track = SPLT_TRUE;
   cu->counter++;
 }
@@ -252,33 +269,89 @@ static void splt_cue_process_index_line(char *line_content, cue_utils *cu, splt_
 //! Process the rest of a cue line that begins with the word REM
 static void splt_cue_process_rem_line(char *line_content, cue_utils *cu, splt_state *state)
 {
-  char *linetail;
+  if (cu->tracks < 0)
+  {
+    return;
+  }
 
-  // Skip the word REM
+  //skip the word REM
   line_content += 3;
 
-  // Skip all leading whitespace after the word REM
-  while ((*line_content==' ')||(*line_content=='\t')) line_content++;
-
-  if((linetail=strstr(line_content,"CREATOR"))!=NULL)
+  //skip all leading whitespace after the word REM
+  while ((*line_content == ' ') || (*line_content == '\t'))
   {
-    // Skip the word "CREATOR"
-    linetail += 7;
+    line_content++;
+  }
 
-    if(strstr(linetail,"MP3SPLT_GTK")!=NULL)
+  if (strstr(line_content, "NOKEEP") != NULL)
+  {
+    cu->current_track_type = SPLT_SKIPPOINT;
+    return;
+  }
+
+  int err = SPLT_OK;
+
+  if (strncmp(line_content, "ALBUM", 5) == 0)
+  {
+    line_content += 5;
+    const char *album = splt_cue_parse_value(line_content, SPLT_FALSE);
+    err = splt_tu_set_tags_field(state, cu->tracks-1, SPLT_TAGS_ALBUM, album);
+    if (err != SPLT_OK) { cu->error = err; return; }
+    return;
+  }
+
+  if (strncmp(line_content, "GENRE", 5) == 0)
+  {
+    line_content += 5;
+    const char *genre = splt_cue_parse_value(line_content, SPLT_FALSE);
+    err = splt_tu_set_tags_field(state, cu->tracks-1, SPLT_TAGS_GENRE, genre);
+    if (err != SPLT_OK) { cu->error = err; return; }
+    return;
+  }
+
+  if (strncmp(line_content, "DATE", 4) == 0)
+  {
+    line_content += 4;
+    const char *year = splt_cue_parse_value(line_content, SPLT_FALSE);
+    err = splt_tu_set_tags_field(state, cu->tracks-1, SPLT_TAGS_YEAR, year);
+    if (err != SPLT_OK) { cu->error = err; return; }
+    return;
+  }
+
+  if (strncmp(line_content, "TRACK", 5) == 0)
+  {
+    line_content += 5;
+    const char *track = splt_cue_parse_value(line_content, SPLT_FALSE);
+    int tracknumber = atoi(track);
+    err = splt_tu_set_tags_field(state, cu->tracks-1, SPLT_TAGS_TRACK, &tracknumber);
+    if (err != SPLT_OK) { cu->error = err; return; }
+    return;
+  }
+
+  if (strncmp(line_content, "COMMENT", 7) == 0)
+  {
+    line_content += 7;
+    const char *comment = splt_cue_parse_value(line_content, SPLT_FALSE);
+    err = splt_tu_set_tags_field(state, cu->tracks-1, SPLT_TAGS_COMMENT, comment);
+    if (err != SPLT_OK) { cu->error = err; return; }
+    return;
+  }
+
+
+  if (strncmp(line_content, "NAME", 4) == 0)
+  {
+    line_content += 4;
+    const char *name = splt_cue_parse_value(line_content, SPLT_FALSE);
+    if (name != NULL)
     {
-      cu->file_has_been_created_by_us = SPLT_TRUE;
+      if (cu->current_name)
+      {
+        free(cu->current_name);
+        cu->current_name = NULL;
+      }
+      cu->current_name = strdup(name);
     }
   }
-  else if((linetail=strstr(line_content,"SPLT_TITLE_IS_FILENAME"))!=NULL)
-  {
-    cu->title_is_filename = SPLT_TRUE;
-  }
-  else if((linetail=strstr(line_content,"NOKEEP"))!=NULL)
-  {
-    if (cu->tracks >= 0)
-      cu->current_track_type=SPLT_SKIPPOINT;
-  }
 }
 
 //! Process the rest of a cue line that begins with the word FILE
@@ -289,7 +362,7 @@ static void splt_cue_process_file_line(char *line_content, cue_utils *cu, splt_s
     return;
   }
 
-  // Skip the word FILE
+  //skip the word FILE
   line_content += 4;
 
   int err = SPLT_OK;
@@ -385,14 +458,20 @@ static cue_utils *splt_cue_cu_new(int *error)
 
   cu->tracks = -1;
   cu->time_for_track = SPLT_TRUE;
-  cu->performer = SPLT_FALSE;
-  cu->title = SPLT_FALSE;
   cu->counter = 0;
   cu->file = NULL;
   cu->error = SPLT_OK;
   cu->current_track_type = SPLT_SPLITPOINT;
-  cu->title_is_filename = SPLT_FALSE;
-  cu->file_has_been_created_by_us = SPLT_FALSE;
+  cu->current_name = NULL;
+
+  int err = SPLT_OK;
+  cu->all_tags = splt_tu_new_tags(&err);
+  if (err < 0)
+  {
+    *error = err;
+    splt_cue_cu_free(&cu);
+    return NULL;
+  }
 
   return cu;
 }
@@ -409,19 +488,18 @@ static void splt_cue_cu_free(cue_utils **cu)
     return;
   }
 
+  if ((*cu)->current_name)
+  {
+    free((*cu)->current_name);
+    (*cu)->current_name = NULL;
+  }
+
+  splt_tu_free_one_tags(&(*cu)->all_tags);
+
   free(*cu);
   *cu = NULL;
 }
 
-/*! Read in split points from a cue file
-
-  \param state the splt_state structure the split points have to be
-  output to.
-  \param error Contains the error code if anything goes wrong
-  \param file The name of the file we have to analyze
-  \todo REM Genre support
-
- */
 int splt_cue_put_splitpoints(const char *file, splt_state *state, int *error)
 {
   if (file == NULL)
@@ -432,7 +510,7 @@ int splt_cue_put_splitpoints(const char *file, splt_state *state, int *error)
   }
 
   splt_c_put_info_message_to_client(state, 
-      _(" reading informations from CUE file %s ...\n"),file);
+      _(" reading informations from CUE file %s ...\n"), file);
 
   splt_t_free_splitpoints_tags(state);
 
@@ -448,15 +526,7 @@ int splt_cue_put_splitpoints(const char *file, splt_state *state, int *error)
   if (err < 0) { *error = err; return tracks; }
   cu->file = file;
 
-  //TODO: .cue #REM GENRE support
-  err = splt_tu_set_tags_field(state, 0, SPLT_TAGS_GENRE, SPLT_UNDEFINED_GENRE);
-  if (err != SPLT_OK)
-  {
-    *error = err;
-    return tracks;
-  }
-
-  if (!(file_input=splt_io_fopen(file, "r")))
+  if (!(file_input = splt_io_fopen(file, "r")))
   {
     splt_e_set_strerror_msg_with_data(state, file);
     *error = SPLT_ERROR_CANNOT_OPEN_FILE;
@@ -479,13 +549,6 @@ int splt_cue_put_splitpoints(const char *file, splt_state *state, int *error)
     if (cu->error < 0) { *error = cu->error; goto function_end; }
   }
 
-  // Append a split point at the end of the file
-  // If the file has been created by us this has already been done
-  // and we can skip this step.
-  if(!cu->file_has_been_created_by_us)
-    err = splt_sp_append_splitpoint(state, LONG_MAX,
-				    _("description here"), cu->current_track_type);
-  
   if (cu->counter == 0)
   {
     splt_e_set_error_data(state, file);
@@ -498,25 +561,10 @@ int splt_cue_put_splitpoints(const char *file, splt_state *state, int *error)
     tracks--;
   }
 
-  // Generate filenames using the tags we got
-  splt_cc_put_filenames_from_tags(state, tracks, error);
-  
-  // If mp3splt_gtk has generated the cue file 
-  // splt_cc_put_filenames_from_tags has now auto-generated
-  // filenames that contain the intended filename (that was
-  // written to the TITLE tag).
-  // Which means we have to correct this by copying the
-  // TITLE tags to the filenames.
-
-    if(cu->title_is_filename)
-      {
-    	int i;
-    	for(i=0;i<tracks;i++)
-    	  {
-    	    splt_sp_set_splitpoint_name(state, i,
-    					splt_tu_get_tags_field(state, i, SPLT_TAGS_TITLE));
-    	  }
-      }
+  if (!splt_o_get_int_option(state, SPLT_OPT_CUE_SET_SPLITPOINT_NAMES_FROM_REM_NAME))
+  {
+    splt_cc_put_filenames_from_tags(state, tracks, error, cu->all_tags);
+  }
 
 function_end:
   splt_cue_cu_free(&cu);
@@ -564,38 +612,78 @@ static void splt_cue_write_title_performer(splt_state *state, FILE *file_output,
     tags = splt_tu_get_current_tags(state);
   }
 
-  if (tags)
-  {
-    if (write_album)
-    {
-      if (tags->album)
-      {
-        if (with_spaces) { fprintf(file_output, "    "); }
-        fprintf(file_output, "TITLE \"%s\"\n", tags->album);
-      }
-    }
-    else
-    {
-      if (tags->title)
-      {
-        if (with_spaces) { fprintf(file_output, "    "); }
-        fprintf(file_output, "TITLE \"%s\"\n", tags->title);
-      }
-    }
-
-    char *performer = splt_tu_get_artist_or_performer_ptr(tags);
-    if (performer)
-    {
-      if (with_spaces) { fprintf(file_output, "    "); }
-      fprintf(file_output, "PERFORMER \"%s\"\n", performer);
-    }
-  }
-  else
+  if (!tags)
   {
     if (with_spaces) { fprintf(file_output, "    "); }
     fprintf(file_output, "TITLE \"\"\n");
     if (with_spaces) { fprintf(file_output, "    "); }
     fprintf(file_output, "PERFORMER \"\"\n");
+    return;
+  }
+
+  if (write_album)
+  {
+    const char *album = splt_tu_get_tags_value(tags, SPLT_TAGS_ALBUM);
+    if (album)
+    {
+      if (with_spaces) { fprintf(file_output, "    "); }
+      fprintf(file_output, "TITLE \"%s\"\n", album);
+    }
+  }
+  else
+  {
+    const char *title = splt_tu_get_tags_value(tags, SPLT_TAGS_TITLE);
+    if (title)
+    {
+      if (with_spaces) { fprintf(file_output, "    "); }
+      fprintf(file_output, "TITLE \"%s\"\n", title);
+    }
+  }
+
+  char *performer = splt_tu_get_artist_or_performer_ptr(tags);
+  if (performer)
+  {
+    if (with_spaces) { fprintf(file_output, "    "); }
+    fprintf(file_output, "PERFORMER \"%s\"\n", performer);
+  }
+}
+
+static void splt_cue_write_other_tags(splt_state *state, FILE *file_output)
+{
+  splt_tags *tags = splt_tu_get_current_tags(state);
+  if (!tags) { return; }
+
+  const char *album = splt_tu_get_tags_value(tags, SPLT_TAGS_ALBUM);
+  if (album != NULL && (strcmp(album, "") != 0))
+  {
+    fprintf(file_output, "    REM ALBUM \"%s\"\n", album);
+  }
+
+  const char *genre = splt_tu_get_tags_value(tags, SPLT_TAGS_GENRE);
+  if (genre != NULL && (strcmp(genre, "") != 0))
+  {
+    fprintf(file_output, "    REM GENRE \"%s\"\n", genre);
+  }
+
+  const char *year = splt_tu_get_tags_value(tags, SPLT_TAGS_YEAR);
+  if (year != NULL && (strcmp(year, "") != 0))
+  {
+    fprintf(file_output, "    REM DATE \"%s\"\n", year);
+  }
+
+  if (!tags->was_auto_incremented)
+  {
+    const int *track = splt_tu_get_tags_value(tags, SPLT_TAGS_TRACK);
+    if (track != NULL && *track > 0)
+    {
+      fprintf(file_output, "    REM TRACK \"%d\"\n", *track);
+    }
+  }
+
+  const char *comment = splt_tu_get_tags_value(tags, SPLT_TAGS_COMMENT);
+  if (comment != NULL && (strcmp(comment, "") != 0))
+  {
+    fprintf(file_output, "    REM COMMENT \"%s\"\n", comment);
   }
 }
 
@@ -614,12 +702,6 @@ void splt_cue_export_to_file(splt_state *state, const char *out_file,
 {
   int err = SPLT_OK;
 
-  int num_of_splitpoints = splt_t_get_splitnumber(state);
-  if (num_of_splitpoints <= 0)
-  {
-    return;
-  }
-
   long total_time = splt_t_get_total_time(state);
   FILE *file_output = NULL;
 
@@ -646,23 +728,39 @@ void splt_cue_export_to_file(splt_state *state, const char *out_file,
 
   char *fname = splt_t_get_filename_to_split(state);
 
-  char new_upper_ext[10] = { '\0' };
-  const char *upper_ext = splt_p_get_upper_extension(state, &err);
-  int i = 0;
-  for (i = 1;i < strlen(upper_ext);i++)
+  const char *extension = strrchr(fname, '.');
+  if (extension != NULL)
   {
-    new_upper_ext[i-1] = upper_ext[i];
+    char upper_extension[5] = { '\0' };
+    int i = 1;
+    for (i = 1;i < strlen(extension);i++)
+    {
+      upper_extension[i-1] = (char) toupper((int)extension[i]);
+    }
+
+    fprintf(file_output, "FILE \"%s\" %s\n", fname, upper_extension);
+    if (err < 0) { *error = err; goto end; }
+  }
+  else
+  {
+    fprintf(file_output, "FILE \"%s\"\n", fname);
+    if (err < 0) { *error = err; goto end; }
   }
 
-  fprintf(file_output, "FILE \"%s\" %s\n", fname, new_upper_ext);
-  if (err < 0) { *error = err; goto end; }
-
   splt_t_set_current_split(state, 0);
+  int num_of_splitpoints = splt_sp_get_real_splitpoints_number(state);
+  int i;
   for (i = 0;i < num_of_splitpoints;i++)
   {
     long splitpoint = splt_sp_get_splitpoint_value(state, i, &err);
     if (err < 0) { *error = err; break; }
 
+    int splitpoint_type = splt_sp_get_splitpoint_type(state, i, &err);
+    if (err < 0) { *error = err; break; }
+
+    const char *name = splt_sp_get_splitpoint_name(state, i, &err);
+    if (err < 0) { *error = err; break; }
+
     //todo: splitpoint can be slightly != than total_time sometimes
     // (test with silence and cue)
     if (stop_at_total_time &&
@@ -675,19 +773,29 @@ void splt_cue_export_to_file(splt_state *state, const char *out_file,
 
     splt_cue_write_title_performer(state, file_output, -1, SPLT_TRUE, SPLT_FALSE);
 
+    splt_cue_write_other_tags(state, file_output);
+
+    if (name != NULL)
+    {
+      fprintf(file_output, "    REM NAME \"%s\"\n", name);
+    }
+
+    if (splitpoint_type == SPLT_SKIPPOINT)
+    {
+      fprintf(file_output, "    REM NOKEEP\n");
+    }
+
     long mins = 0, secs = 0, hundr = 0;
     if (splitpoint == LONG_MAX)
     {
       splitpoint = total_time;
     }
-
     splt_sp_get_mins_secs_hundr_from_splitpoint(splitpoint, &mins, &secs, &hundr);
     fprintf(file_output, "    INDEX 01 %02ld:%02ld:%02ld\n", mins, secs, hundr);
 
-    splt_t_current_split_next(state);
+    splt_t_set_current_split_file_number_next(state);
   }
 
-end:
   fflush(file_output);
   if (fclose(file_output) != 0)
   {
@@ -696,8 +804,11 @@ end:
   }
   file_output = NULL;
 
-  splt_c_put_info_message_to_client(state, 
-      _(" CUE file '%s' created.\n"), cue_out_file);
+end:
+  if (!splt_o_get_int_option(state, SPLT_OPT_CUE_DISABLE_CUE_FILE_CREATED_MESSAGE))
+  {
+    splt_c_put_info_message_to_client(state, _(" CUE file '%s' created.\n"), cue_out_file);
+  }
 
   if (cue_out_file)
   {
