@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2012 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2013 Alexandru Munteanu - m@ioalex.net
  *
  * http://mp3splt.sourceforge.net
  *
@@ -24,8 +24,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307,
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
  * USA.
  *
  *********************************************************/
@@ -39,7 +38,6 @@ const char *splt_p_get_upper_extension(splt_state *state, int *error);
 int splt_p_check_plugin_is_for_file(splt_state *state, int *error);
 void splt_p_search_syncerrors(splt_state *state, int *error);
 void splt_p_dewrap(splt_state *state, int listonly, const char *dir, int *error);
-void splt_p_set_total_time(splt_state *state, int *error);
 double splt_p_split(splt_state *state, const char *final_fname, double begin_point,
     double end_point, int *error, int save_end_point);
 int splt_p_simple_split(splt_state *state, const char *output_fname, off_t begin,
