@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2012 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2013 Alexandru Munteanu - m@ioalex.net
  *
  * http://mp3splt.sourceforge.net
  *
@@ -24,8 +24,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307,
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
  * USA.
  *
  *********************************************************/
@@ -56,8 +55,8 @@ static void splt_tp_process_auto_increment_tracknumber_variable(
 static void splt_tp_check_ambigous_next_position(const char *tag_variable_start,
     tags_parser_utils *tpu);
 static void splt_tp_process_original_tags_variable(tags_parser_utils *tpu,
-    splt_state *state, int *error);
-static void splt_tp_get_original_tags(splt_state *state, int *error);
+    splt_state *state, int *error, int set_original_tags);
+static void splt_tp_get_original_tags_and_append(splt_state *state, int *error);
 static int splt_tp_tpu_has_one_current_tag_set(tags_parser_utils *tpu);
 static char *splt_tp_look_for_end_paranthesis(tags_parser_utils *tpu);
 static void splt_tp_look_for_all_tags_char(const char *tags,
@@ -194,7 +193,7 @@ int splt_tp_put_tags_from_string(splt_state *state, const char *tags, int *error
     if (*error < 0) { goto end; }
 
     splt_tu_append_tags_to_state(state, tpu->current_tags,
-        !tpu->original_tags_found, error);
+        !tpu->original_tags_found, tpu->original_tags_value, SPLT_FALSE, error);
     if (*error < 0) { goto end; }
 
     if (tpu->set_all_tags)
@@ -206,7 +205,13 @@ int splt_tp_put_tags_from_string(splt_state *state, const char *tags, int *error
     }
     else if (tpu->we_had_all_tags && !tpu->original_tags_found)
     {
-      int index = state->split.real_tagsnumber - 1;
+      int real_tags_number = 0;
+      if (state->split.tags_group)
+      {
+        real_tags_number = state->split.tags_group->real_tagsnumber;
+      }
+
+      int index = real_tags_number - 1;
       splt_tu_set_new_tags_where_current_tags_are_null(state, 
           tpu->current_tags, tpu->all_tags, index, error);
       if (*error < 0) { goto end; }
@@ -307,6 +312,13 @@ static void splt_tp_set_track_from_parsed_tracknumber(tags_parser_utils *tpu,
 {
   const char *tracknumber = splt_tp_tpu_get_tracknumber(tpu);
 
+  if (tracknumber != NULL && strcmp(tracknumber, "-2") == 0)
+  {
+    int track = -2;
+    splt_tp_tpu_set_tags_value(tpu, SPLT_TAGS_TRACK, &track);
+    return;
+  }
+
   if (tracknumber)
   {
     int is_number = SPLT_TRUE;
@@ -344,7 +356,11 @@ static void splt_tp_process_tag_variable(const char *tag_variable_start,
   switch (tag_variable)
   {
     case 'o':
-      splt_tp_process_original_tags_variable(tpu, state, error);
+      splt_tp_process_original_tags_variable(tpu, state, error, SPLT_TRUE);
+      if (*error < 0) { return; }
+      break;
+    case 'O':
+      splt_tp_process_original_tags_variable(tpu, state, error, SAME_BYTES_AS_TAGS);
       if (*error < 0) { return; }
       break;
     case 'a':
@@ -481,7 +497,7 @@ static void splt_tp_check_ambigous_next_position(const char *tag_variable_start,
 }
 
 static void splt_tp_process_original_tags_variable(tags_parser_utils *tpu,
-    splt_state *state, int *error)
+    splt_state *state, int *error, int set_original_tags)
 {
   if (tpu->original_tags_found)
   {
@@ -495,21 +511,20 @@ static void splt_tp_process_original_tags_variable(tags_parser_utils *tpu,
 
   splt_o_lock_messages(state);
 
-  splt_tp_get_original_tags(state, error);
+  splt_tp_get_original_tags_and_append(state, error);
   if (*error < 0) { goto end; }
 
-  int err = splt_tu_append_original_tags(state);
-  if (err < 0) { *error = err; goto end; }
+  splt_tags appended_tags = splt_tu_get_last_tags(state);
+  splt_tu_set_field_on_tags(&appended_tags, SPLT_TAGS_ORIGINAL, &set_original_tags);
 
   if (tpu->set_all_tags)
   {
-    splt_tags last_tags = splt_tu_get_last_tags(state);
-    splt_tu_copy_tags(&last_tags, tpu->all_tags, error);
-    tpu->all_tags->set_original_tags = SPLT_TRUE;
+    splt_tu_copy_tags(&appended_tags, tpu->all_tags, error);
     if (*error < 0) { goto end; }
   }
 
   tpu->original_tags_found = SPLT_TRUE;
+  tpu->original_tags_value = set_original_tags;
 
   if (splt_tp_tpu_has_one_current_tag_set(tpu))
   {
@@ -520,7 +535,7 @@ end:
   splt_o_unlock_messages(state);
 }
 
-static void splt_tp_get_original_tags(splt_state *state, int *error)
+static void splt_tp_get_original_tags_and_append(splt_state *state, int *error)
 {
   splt_check_file_type(state, error);
   if (*error < 0) { return; }
@@ -532,6 +547,9 @@ static void splt_tp_get_original_tags(splt_state *state, int *error)
 
   splt_tu_get_original_tags(state, error);
 
+  int err = splt_tu_append_original_tags(state);
+  if (err < 0) { *error = err; }
+
   splt_p_end(state, error);
 
   return;
@@ -586,10 +604,10 @@ static tags_parser_utils *splt_tp_tpu_new(splt_state *state, int *error)
   tpu->set_all_tags = SPLT_FALSE;
   tpu->we_had_all_tags = SPLT_FALSE;
 
-  tpu->all_tags = splt_tu_new_tags(state, error);
+  tpu->all_tags = splt_tu_new_tags(error);
   if (*error < 0) { goto mem_error; }
 
-  tpu->current_tags = splt_tu_new_tags(state, error);
+  tpu->current_tags = splt_tu_new_tags(error);
   if (*error < 0) { goto mem_error; }
 
   tpu->current_tracknumber = NULL;
@@ -622,6 +640,7 @@ static void splt_tp_tpu_reset_for_new_tags(splt_state *state, tags_parser_utils
   splt_tu_free_one_tags_content(tpu->current_tags);
 
   tpu->original_tags_found = SPLT_FALSE;
+  tpu->original_tags_value = SPLT_FALSE;
 
   if (tpu->current_tracknumber)
   {
